#ifndef TRACE_COMMON
#define TRACE_COMMON

#include <linux/proc_fs.h>
#include <linux/types.h>
#include <linux/string.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/spinlock.h>

#include <linux/dirent.h>
#include <asm/statfs.h>
#include <linux/utime.h>
#include <asm/uaccess.h>
#include <linux/uio.h>
#include <linux/kprobes.h>

#define PROCFS_DIRECTORY_NAME "io_tracer"
#define PROCFS_MAX_SIZE 1024
#define BUFFER_CHAIN_SIZE 2056

/* linux_dirent is defined in 
   readdir.c but must be forward
   declared here.  syscalls.h also
   must forward declare this */
struct linux_dirent;

int procfile_trace_read(char* buffer,
			char** buffer_location,
			off_t offset, 
			int buffer_length, 
			int* eof, 
			void* data);
int procfile_trace_write(struct file *file, 
			 const char *buffer, 
			 unsigned long count, 
			 void *data);

extern struct proc_dir_entry* proc_directory;

typedef struct kprobes_locks
{
  int locked;
  spinlock_t lock;
  unsigned long flags;
} kprobes_locks;

/* trace simplification functions */
int ret_handler(void);
long entry_handler(void);


/* Each io call should have a jprobe, a kretprobe, a
   jprobe handler, a kretprobe handler, and a unique
   index between 0 and 127.
   In the event that there are more than 128 things being
   traced, the size of the int array a_krpobes must be changed. */

extern struct jprobe jp_sys_open;
extern struct kretprobe krp_sys_open;
asmlinkage long trace_sys_open(const char __user * filename, 
			       int flags, 
			       int mode);
int ret_handler_open(struct kretprobe_instance *ri, 
		     struct pt_regs *regs);
#define TRACE_OPEN 1

extern struct jprobe jp_sys_close;
extern struct kretprobe krp_sys_close;
asmlinkage long trace_sys_close(unsigned int fd);
int ret_handler_close(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_CLOSE 2

extern struct jprobe jp_sys_bdflush;
extern struct kretprobe krp_sys_bdflush;
asmlinkage long trace_sys_bdflush(int func, long data);
int ret_handler_bdflush(struct kretprobe_instance *ri, 
			struct pt_regs *regs);
#define TRACE_BDFLUSH 3

extern struct jprobe jp_sys_creat;
extern struct kretprobe krp_sys_creat;
asmlinkage long trace_sys_creat(const char __user* pathname,
				int mode);
int ret_handler_creat(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_CREAT 4

extern struct jprobe jp_sys_access;
extern struct kretprobe krp_sys_access;
asmlinkage long trace_sys_access(const char __user* filename,
				 int mode);
int ret_handler_access(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_ACCESS 5

extern struct jprobe jp_sys_read;
extern struct kretprobe krp_sys_read;
asmlinkage long trace_sys_read(unsigned int fd,
			       char __user* buf,
			       size_t count);
int ret_handler_read(struct kretprobe_instance *ri, 
		     struct pt_regs *regs);
#define TRACE_READ 6

extern struct jprobe jp_sys_readahead;
extern struct kretprobe krp_sys_readahead;
asmlinkage long trace_sys_readahead(int fd,
			       loff_t offset,
			       size_t count);
int ret_handler_readahead(struct kretprobe_instance *ri, 
			  struct pt_regs *regs);
#define TRACE_READAHEAD 7

extern struct jprobe jp_sys_readv;
extern struct kretprobe krp_sys_readv;
asmlinkage long trace_sys_readv(unsigned long fd,
			       const struct iovec __user* vec,
			       unsigned long vlen);
int ret_handler_readv(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_READV 8

extern struct jprobe jp_sys_readlink;
extern struct kretprobe krp_sys_readlink;
asmlinkage long trace_sys_readlink(const char __user* pathname,
				   char __user* buf,
				   int bufsize);
int ret_handler_readlink(struct kretprobe_instance *ri, 
			 struct pt_regs *regs);
#define TRACE_READLINK 9

extern struct jprobe jp_sys_pread64;
extern struct kretprobe krp_sys_pread64;
asmlinkage long trace_sys_pread64(unsigned int fd,
				  char __user* buf,
				  size_t count,
				  loff_t pos);
int ret_handler_pread64(struct kretprobe_instance *ri, 
			struct pt_regs *regs);
#define TRACE_PREAD64 10

extern struct jprobe jp_sys_chmod;
extern struct kretprobe krp_sys_chmod;
asmlinkage long trace_sys_chmod(const char __user* filename, 
				mode_t mode);
int ret_handler_chmod(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_CHMOD 11

extern struct jprobe jp_sys_fchmod;
extern struct kretprobe krp_sys_fchmod;
asmlinkage long trace_sys_fchmod(unsigned int fd, mode_t mode);
int ret_handler_fchmod(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_FCHMOD 12

extern struct jprobe jp_sys_chown;
extern struct kretprobe krp_sys_chown;
asmlinkage long trace_sys_chown(const char __user* filename, 
				uid_t user,
				gid_t group);
int ret_handler_chown(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_CHOWN 13

extern struct jprobe jp_sys_lchown;
extern struct kretprobe krp_sys_lchown;
asmlinkage long trace_sys_lchown(const char __user* filename, 
				 uid_t user,
				 gid_t group);
int ret_handler_lchown(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_LCHOWN 14

extern struct jprobe jp_sys_fchown;
extern struct kretprobe krp_sys_fchown;
asmlinkage long trace_sys_fchown(const char __user* filename, 
				 uid_t user,
				 gid_t group);
int ret_handler_fchown(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_FCHOWN 15

extern struct jprobe jp_sys_link;
extern struct kretprobe krp_sys_link;
asmlinkage long trace_sys_link(const char __user* oldname, 
			       const char __user* newname);
int ret_handler_link(struct kretprobe_instance *ri, 
		     struct pt_regs *regs);
#define TRACE_LINK 16

extern struct jprobe jp_sys_symlink;
extern struct kretprobe krp_sys_symlink;
asmlinkage long trace_sys_symlink(const char __user* old, 
				  const char __user* new);
int ret_handler_symlink(struct kretprobe_instance *ri, 
			struct pt_regs *regs);
#define TRACE_SYMLINK 17

extern struct jprobe jp_sys_unlink;
extern struct kretprobe krp_sys_unlink;
asmlinkage long trace_sys_unlink(const char __user* pathname);
int ret_handler_unlink(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_UNLINK 18

extern struct jprobe jp_sys_sync;
extern struct kretprobe krp_sys_sync;
asmlinkage long trace_sys_sync(void);
int ret_handler_sync(struct kretprobe_instance *ri, 
		     struct pt_regs *regs);
#define TRACE_SYNC 19

extern struct jprobe jp_sys_fsync;
extern struct kretprobe krp_sys_fsync;
asmlinkage long trace_sys_fsync(unsigned int fd);
int ret_handler_fsync(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_FSYNC 20

extern struct jprobe jp_sys_fdatasync;
extern struct kretprobe krp_sys_fdatasync;
asmlinkage long trace_sys_fdatasync(unsigned int fd);
int ret_handler_fdatasync(struct kretprobe_instance *ri, 
			  struct pt_regs *regs);
#define TRACE_FDATASYNC 21

extern struct jprobe jp_sys_mount;
extern struct kretprobe krp_sys_mount;
asmlinkage long trace_sys_mount(char __user* dev_name,
				char __user* dir_name,
				char __user* type,
				unsigned long flags,
				void __user* data);
int ret_handler_mount(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_MOUNT 22

extern struct jprobe jp_sys_umount;
extern struct kretprobe krp_sys_umount;
asmlinkage long trace_sys_umount(char __user* name, int flags);
int ret_handler_umount(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_UMOUNT 23

extern struct jprobe jp_sys_oldumount;
extern struct kretprobe krp_sys_oldumount;
asmlinkage long trace_sys_oldumount(char __user* name);
int ret_handler_oldumount(struct kretprobe_instance *ri, 
			  struct pt_regs *regs);
#define TRACE_OLDUMOUNT 24

extern struct jprobe jp_sys_stat;
extern struct kretprobe krp_sys_stat;
asmlinkage long trace_sys_stat(const char __user * filename, 
			       struct __old_kernel_stat __user* statbuf);
int ret_handler_stat(struct kretprobe_instance *ri, 
		     struct pt_regs *regs);
#define TRACE_STAT 25

extern struct jprobe jp_sys_statfs;
extern struct kretprobe krp_sys_statfs;
asmlinkage long trace_sys_statfs(const char __user* path, 
				 struct statfs __user* buf);
int ret_handler_statfs(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_STATFS 26

extern struct jprobe jp_sys_statfs64;
extern struct kretprobe krp_sys_statfs64;
asmlinkage long trace_sys_statfs64(const char __user* path, 
				   struct statfs64 __user* buf);
int ret_handler_statfs64(struct kretprobe_instance *ri, 
			 struct pt_regs *regs);
#define TRACE_STATFS64 27

extern struct jprobe jp_sys_fstatfs;
extern struct kretprobe krp_sys_fstatfs;
asmlinkage long trace_sys_fstatfs(unsigned int fd, 
				  struct statfs __user* buf);
int ret_handler_fstatfs(struct kretprobe_instance *ri, 
			struct pt_regs *regs);
#define TRACE_FSTATFS 28

extern struct jprobe jp_sys_fstatfs64;
extern struct kretprobe krp_sys_fstatfs64;
asmlinkage long trace_sys_fstatfs64(unsigned int fd, 
				    size_t sz, 
				    struct statfs64 __user* buf);
int ret_handler_fstatfs64(struct kretprobe_instance *ri, 
			  struct pt_regs *regs);
#define TRACE_FSTATFS64 29

extern struct jprobe jp_sys_lstat;
extern struct kretprobe krp_sys_lstat;
asmlinkage long trace_sys_lstat(const char __user * filename, 
				struct __old_kernel_stat __user* statbuf);
int ret_handler_lstat(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_LSTAT 30

extern struct jprobe jp_sys_fstat;
extern struct kretprobe krp_sys_fstat;
asmlinkage long trace_sys_fstat(unsigned int fd, 
				struct __old_kernel_stat __user* statbuf);
int ret_handler_fstat(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_FSTAT 31

extern struct jprobe jp_sys_newstat;
extern struct kretprobe krp_sys_newstat;
asmlinkage long trace_sys_newstat(const char __user * filename, 
				  struct stat __user* statbuf);
int ret_handler_newstat(struct kretprobe_instance *ri, 
			struct pt_regs *regs);
#define TRACE_NEWSTAT 32

extern struct jprobe jp_sys_newlstat;
extern struct kretprobe krp_sys_newlstat;
asmlinkage long trace_sys_newlstat(const char __user * filename, 
				   struct stat __user* statbuf);
int ret_handler_newlstat(struct kretprobe_instance *ri, 
			 struct pt_regs *regs);
#define TRACE_NEWLSTAT 33

extern struct jprobe jp_sys_newfstat;
extern struct kretprobe krp_sys_newfstat;
asmlinkage long trace_sys_newfstat(unsigned int fd, 
				   struct stat __user* statbuf);
int ret_handler_newfstat(struct kretprobe_instance *ri, 
			 struct pt_regs *regs);
#define TRACE_NEWFSTAT 34

extern struct jprobe jp_sys_ustat;
extern struct kretprobe krp_sys_ustat;
asmlinkage long trace_sys_ustat(unsigned dev, struct ustat __user* ubuf);
int ret_handler_ustat(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_USTAT 35

extern struct jprobe jp_sys_truncate;
extern struct kretprobe krp_sys_truncate;
asmlinkage long trace_sys_truncate(const char __user* path, 
				   unsigned long length);
int ret_handler_truncate(struct kretprobe_instance *ri, 
			 struct pt_regs *regs);
#define TRACE_TRUNCATE 36

extern struct jprobe jp_sys_ftruncate;
extern struct kretprobe krp_sys_ftruncate;
asmlinkage long trace_sys_ftruncate(unsigned int fd, 
				    unsigned long length);
int ret_handler_ftruncate(struct kretprobe_instance *ri, 
			  struct pt_regs *regs);
#define TRACE_FTRUNCATE 37

extern struct jprobe jp_sys_setxattr;
extern struct kretprobe krp_sys_setxattr;
asmlinkage long trace_sys_setxattr(char __user* path, 
				   char __user* name,
				   void __user* value, 
				   size_t size,
				   int flags);
int ret_handler_setxattr(struct kretprobe_instance *ri, 
			 struct pt_regs *regs);
#define TRACE_SETXATTR 38

extern struct jprobe jp_sys_lsetxattr;
extern struct kretprobe krp_sys_lsetxattr;
asmlinkage long trace_sys_lsetxattr(char __user* path, 
				    char __user* name,
				    void __user* value, 
				    size_t size,
				    int flags);
int ret_handler_lsetxattr(struct kretprobe_instance *ri, 
			  struct pt_regs *regs);
#define TRACE_LSETXATTR 39

extern struct jprobe jp_sys_fsetxattr;
extern struct kretprobe krp_sys_fsetxattr;
asmlinkage long trace_sys_fsetxattr(int fd, 
				    char __user* name,
				    void __user* value, 
				    size_t size,
				    int flags);
int ret_handler_fsetxattr(struct kretprobe_instance *ri, 
			  struct pt_regs *regs);
#define TRACE_FSETXATTR 40

extern struct jprobe jp_sys_getxattr;
extern struct kretprobe krp_sys_getxattr;
asmlinkage ssize_t trace_sys_getxattr(char __user* path, 
				      char __user* name,
				      void __user* value, 
				      size_t size);
int ret_handler_getxattr(struct kretprobe_instance *ri, 
			 struct pt_regs *regs);
#define TRACE_GETXATTR 41

extern struct jprobe jp_sys_lgetxattr;
extern struct kretprobe krp_sys_lgetxattr;
asmlinkage ssize_t trace_sys_lgetxattr(char __user* path, 
				       char __user* name,
				       void __user* value, 
				       size_t size);
int ret_handler_lgetxattr(struct kretprobe_instance *ri, 
			  struct pt_regs *regs);
#define TRACE_LGETXATTR 42

extern struct jprobe jp_sys_fgetxattr;
extern struct kretprobe krp_sys_fgetxattr;
asmlinkage ssize_t trace_sys_fgetxattr(int fd, 
				       char __user* name,
				       void __user* value, 
				       size_t size,
				       int flags);
int ret_handler_fgetxattr(struct kretprobe_instance *ri, 
			  struct pt_regs *regs);
#define TRACE_FGETXATTR 43

extern struct jprobe jp_sys_listxattr;
extern struct kretprobe krp_sys_listxattr;
asmlinkage ssize_t trace_sys_listxattr(char __user* path,
				       char __user* list,
				       size_t size);
int ret_handler_listxattr(struct kretprobe_instance *ri, 
			  struct pt_regs *regs);
#define TRACE_LISTXATTR 44

extern struct jprobe jp_sys_llistxattr;
extern struct kretprobe krp_sys_llistxattr;
asmlinkage ssize_t trace_sys_llistxattr(char __user* path, 
					char __user* list,
					size_t size);
int ret_handler_llistxattr(struct kretprobe_instance *ri, 
			   struct pt_regs *regs);
#define TRACE_LLISTXATTR 45

extern struct jprobe jp_sys_flistxattr;
extern struct kretprobe krp_sys_flistxattr;
asmlinkage ssize_t trace_sys_flistxattr(int fd, 
					char __user* list, 
					size_t size);
int ret_handler_flistxattr(struct kretprobe_instance *ri, 
			   struct pt_regs *regs);
#define TRACE_FLISTXATTR 46

extern struct jprobe jp_sys_removexattr;
extern struct kretprobe krp_sys_removexattr;
asmlinkage long trace_sys_removexattr(char __user* path,
				      char __user* name);
int ret_handler_removexattr(struct kretprobe_instance *ri, 
			    struct pt_regs *regs);
#define TRACE_REMOVEXATTR 47

extern struct jprobe jp_sys_lremovexattr;
extern struct kretprobe krp_sys_lremovexattr;
asmlinkage long trace_sys_lremovexattr(char __user* path,
				       char __user* name);
int ret_handler_lremovexattr(struct kretprobe_instance *ri, 
			     struct pt_regs *regs);
#define TRACE_LREMOVEXATTR 48

extern struct jprobe jp_sys_fremovexattr;
extern struct kretprobe krp_sys_fremovexattr;
asmlinkage long trace_sys_fremovexattr(int fd, 
				       char __user* name);
int ret_handler_fremovexattr(struct kretprobe_instance *ri, 
			     struct pt_regs *regs);
#define TRACE_FREMOVEXATTR 49

extern struct jprobe jp_sys_brk;
extern struct kretprobe krp_sys_brk;
asmlinkage unsigned long trace_sys_brk(unsigned long brk);
int ret_handler_brk(struct kretprobe_instance *ri, 
		    struct pt_regs *regs);
#define TRACE_BRK 50

extern struct jprobe jp_sys_rename;
extern struct kretprobe krp_sys_rename;
asmlinkage long trace_sys_rename(const char __user* oldname,
				 const char __user* newname);
int ret_handler_rename(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_RENAME 51

extern struct jprobe jp_sys_mknod;
extern struct kretprobe krp_sys_mknod;
asmlinkage long trace_sys_mknod(const char __user* filename,
				int mode,
				unsigned dev);
int ret_handler_mknod(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_MKNOD 52

extern struct jprobe jp_sys_fcntl;
extern struct kretprobe krp_sys_fcntl;
asmlinkage long trace_sys_fcntl(unsigned int fd,
				unsigned int cmd,
				unsigned long arg);
int ret_handler_fcntl(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_FCNTL 53

extern struct jprobe jp_sys_dup;
extern struct kretprobe krp_sys_dup;
asmlinkage long trace_sys_dup(unsigned int fildes);
int ret_handler_dup(struct kretprobe_instance *ri, 
		    struct pt_regs *regs);
#define TRACE_DUP 54

extern struct jprobe jp_sys_dup2;
extern struct kretprobe krp_sys_dup2;
asmlinkage long trace_sys_dup2(unsigned int oldfd, 
			       unsigned int newfd);
int ret_handler_dup2(struct kretprobe_instance *ri, 
		     struct pt_regs *regs);
#define TRACE_DUP2 55

extern struct jprobe jp_sys_ioctl;
extern struct kretprobe krp_sys_ioctl;
asmlinkage long trace_sys_ioctl(unsigned int fd,
				unsigned int cmd,
				unsigned long arg);
int ret_handler_ioctl(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_IOCTL 56

extern struct jprobe jp_sys_flock;
extern struct kretprobe krp_sys_flock;
asmlinkage long trace_sys_flock(unsigned int fd,
				unsigned int cmd);
int ret_handler_flock(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_FLOCK 57

extern struct jprobe jp_sys_getcwd;
extern struct kretprobe krp_sys_getcwd;
asmlinkage long trace_sys_getcwd(char __user* buf,
				 unsigned long size);
int ret_handler_getcwd(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_GETCWD 58

extern struct jprobe jp_sys_mkdir;
extern struct kretprobe krp_sys_mkdir;
asmlinkage long trace_sys_mkdir(const char __user* pathname,
				int mode);
int ret_handler_mkdir(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_MKDIR 59

extern struct jprobe jp_sys_chdir;
extern struct kretprobe krp_sys_chdir;
asmlinkage long trace_sys_chdir(const char __user* filename);
int ret_handler_chdir(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_CHDIR 60

extern struct jprobe jp_sys_fchdir;
extern struct kretprobe krp_sys_fchdir;
asmlinkage long trace_sys_fchdir(unsigned int fd);
int ret_handler_fchdir(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_FCHDIR 61

extern struct jprobe jp_sys_rmdir;
extern struct kretprobe krp_sys_rmdir;
asmlinkage long trace_sys_rmdir(const char __user* pathname);
int ret_handler_rmdir(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_RMDIR 62

extern struct jprobe jp_sys_getdents;
extern struct kretprobe krp_sys_getdents;
asmlinkage long trace_sys_getdents(unsigned int fd,
				   struct linux_dirent __user* dirent,
				   unsigned int count);
int ret_handler_getdents(struct kretprobe_instance *ri, 
			 struct pt_regs *regs);
#define TRACE_GETDENTS 63

extern struct jprobe jp_sys_getdents64;
extern struct kretprobe krp_sys_getdents64;
asmlinkage long trace_sys_getdents64(unsigned int fd,
				     struct linux_dirent64 __user* dirent,
				     unsigned int count);
int ret_handler_getdents64(struct kretprobe_instance *ri, 
			   struct pt_regs *regs);
#define TRACE_GETDENTS64 64

extern struct jprobe jp_sys_quotactl;
extern struct kretprobe krp_sys_quotactl;
asmlinkage long trace_sys_quotactl(unsigned int cmd,
				   const char __user* special,
				   qid_t id,
				   void __user* addr);
int ret_handler_quotactl(struct kretprobe_instance *ri, 
			 struct pt_regs *regs);
#define TRACE_QUOTACTL 65

extern struct jprobe jp_sys_write;
extern struct kretprobe krp_sys_write;
asmlinkage long trace_sys_write(unsigned int fd,
				const char __user* buf,
				size_t count);
int ret_handler_write(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_WRITE 66

extern struct jprobe jp_sys_writev;
extern struct kretprobe krp_sys_writev;
asmlinkage long trace_sys_writev(unsigned int fd,
				 const struct iovec __user* vec,
				 unsigned long vlen);
int ret_handler_writev(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_WRITEV 67

extern struct jprobe jp_sys_pwrite64;
extern struct kretprobe krp_sys_pwrite64;
asmlinkage long trace_sys_pwrite64(unsigned int fd,
				   const char __user* buf,
				   size_t count,
				   loff_t pos);
int ret_handler_pwrite64(struct kretprobe_instance *ri, 
			 struct pt_regs *regs);
#define TRACE_PWRITE64 68

extern struct jprobe jp_sys_lseek;
extern struct kretprobe krp_sys_lseek;
asmlinkage long trace_sys_lseek(unsigned int fd,
				off_t offset,
				unsigned int origin);
int ret_handler_lseek(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_LSEEK 69

extern struct jprobe jp_sys_llseek;
extern struct kretprobe krp_sys_llseek;
asmlinkage long trace_sys_llseek(unsigned int fd,
				 unsigned long offset_high,
				 unsigned long offset_low,
				 loff_t __user* result,
				 unsigned int origin);
int ret_handler_llseek(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_LLSEEK 70

extern struct jprobe jp_sys_utime;
extern struct kretprobe krp_sys_utime;
asmlinkage long trace_sys_utime(char __user* filename,
				struct utimbuf __user* times);
int ret_handler_utime(struct kretprobe_instance *ri, 
		      struct pt_regs *regs);
#define TRACE_UTIME 71

extern struct jprobe jp_sys_utimes;
extern struct kretprobe krp_sys_utimes;
asmlinkage long trace_sys_utimes(char __user* filename,
				 struct utimbuf __user* utimes);
int ret_handler_utimes(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_UTIMES 72

extern struct jprobe jp_sys_uselib;
extern struct kretprobe krp_sys_uselib;
asmlinkage long trace_sys_uselib(const char __user* library);
int ret_handler_uselib(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_USELIB 73

#ifdef CONFIG_UID16
extern struct jprobe jp_sys_chown16;
extern struct kretprobe krp_sys_chown16;
asmlinkage long trace_sys_chown16(const char __user* filename, 
				  uid_t user,
				  gid_t group);
int ret_handler_chown16(struct kretprobe_instance *ri, 
			struct pt_regs *regs);
#define TRACE_CHOWN16 74

extern struct jprobe jp_sys_lchown16;
extern struct kretprobe krp_sys_lchown16;
asmlinkage long trace_sys_lchown16(const char __user* filename, 
				   uid_t user,
				   gid_t group);
int ret_handler_lchown16(struct kretprobe_instance *ri, 
			 struct pt_regs *regs);
#define TRACE_LCHOWN16 75

extern struct jprobe jp_sys_fchown16;
extern struct kretprobe krp_sys_fchown16;
asmlinkage long trace_sys_fchown16(const char __user* filename, 
				   uid_t user,
				   gid_t group);
int ret_handler_fchown16(struct kretprobe_instance *ri, 
			 struct pt_regs *regs);
#define TRACE_FCHOWN16 76
#endif

#if BITS_PER_LONG == 32
extern struct jprobe jp_sys_stat64;
extern struct kretprobe krp_sys_stat64;
asmlinkage long trace_sys_stat64(const char __user* filename, 
				struct stat64 __user* statbuf);
int ret_handler_stat64(struct kretprobe_instance *ri, 
		       struct pt_regs *regs);
#define TRACE_STAT64 77

extern struct jprobe jp_sys_fstat64;
extern struct kretprobe krp_sys_fstat64;
asmlinkage long trace_sys_fstat64(unsigned long fd, 
				  struct stat64 __user* statbuf);
int ret_handler_fstat64(struct kretprobe_instance *ri, 
			struct pt_regs *regs);
#define TRACE_FSTAT64 78

extern struct jprobe jp_sys_lstat64;
extern struct kretprobe krp_sys_lstat64;
asmlinkage long trace_sys_lstat64(char __user* filename, 
				  struct stat64 __user* statbuf);
int ret_handler_lstat64(struct kretprobe_instance *ri, 
			struct pt_regs *regs);
#define TRACE_LSTAT64 79

extern struct jprobe jp_sys_truncate64;
extern struct kretprobe krp_sys_truncate64;
asmlinkage long trace_sys_truncate64(const char __user* path, 
				     loff_t length);
int ret_handler_truncate64(struct kretprobe_instance *ri, 
			   struct pt_regs *regs);
#define TRACE_TRUNCATE64 80

extern struct jprobe jp_sys_ftruncate64;
extern struct kretprobe krp_sys_ftruncate64;
asmlinkage long trace_sys_ftruncate64(unsigned int fd, 
				      loff_t length);
int ret_handler_ftruncate64(struct kretprobe_instance *ri, 
			    struct pt_regs *regs);
#define TRACE_FTRUNCATE64 81

extern struct jprobe jp_sys_fcntl64;
extern struct kretprobe krp_sys_fcntl64;
asmlinkage long trace_sys_fcntl64(unsigned int fd,
				  unsigned int cmd,
				  unsigned long arg);
int ret_handler_fcntl64(struct kretprobe_instance *ri, 
			struct pt_regs *regs);
#define TRACE_FCNTL64 82
#endif

#endif /* TRACE_COMMON */
