#include "trace_common.h"

struct buffer_chain_node* head;

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Marlow Weston");
MODULE_DESCRIPTION("System File I/O Tracer");

/***********************************************************************/
typedef struct kprobe_table_entry
{
  char name[32];
  void* entry;
  void* ret;
  struct jprobe* pjp;
  struct kretprobe* pkrp;
  int index;
} kprobe_table_entry;

static struct kprobe_table_entry kprobe_table[] =
{
  {"sys_access", trace_sys_access, ret_handler_access, 
   &jp_sys_access, &krp_sys_access, TRACE_ACCESS},
  
  {"sys_close", trace_sys_close, ret_handler_close, 
   &jp_sys_close, &krp_sys_close, TRACE_CLOSE},

  {"sys_creat", trace_sys_creat, ret_handler_creat, 
   &jp_sys_creat, &krp_sys_creat, TRACE_CREAT},
  
  {"sys_open", trace_sys_open, ret_handler_open, 
   &jp_sys_open, &krp_sys_open, TRACE_OPEN},

  {"sys_chmod", trace_sys_chmod, ret_handler_chmod, 
   &jp_sys_chmod, &krp_sys_chmod, TRACE_CHMOD},
  {"sys_fchmod", trace_sys_fchmod, ret_handler_fchmod, 
   &jp_sys_fchmod, &krp_sys_fchmod, TRACE_FCHMOD},

  {"sys_chown", trace_sys_chown, ret_handler_chown, 
   &jp_sys_chown, &krp_sys_chown, TRACE_CHOWN},
  {"sys_lchown", trace_sys_lchown, ret_handler_lchown, 
   &jp_sys_lchown, &krp_sys_lchown, TRACE_LCHOWN},
  {"sys_fchown", trace_sys_fchown, ret_handler_fchown, 
   &jp_sys_fchown, &krp_sys_fchown, TRACE_FCHOWN},
  
  {"sys_link", trace_sys_link, ret_handler_link, 
   &jp_sys_link, &krp_sys_link, TRACE_LINK},
  {"sys_symlink", trace_sys_symlink, ret_handler_symlink, 
   &jp_sys_symlink, &krp_sys_symlink, TRACE_SYMLINK},
  {"sys_unlink", trace_sys_unlink, ret_handler_unlink, 
   &jp_sys_unlink, &krp_sys_unlink, TRACE_UNLINK},

  {"sys_mount", trace_sys_mount, ret_handler_mount, 
   &jp_sys_mount, &krp_sys_mount, TRACE_MOUNT},
  {"sys_umount", trace_sys_umount, ret_handler_umount, 
   &jp_sys_umount, &krp_sys_umount, TRACE_UMOUNT},
  {"sys_oldumount", trace_sys_oldumount, ret_handler_oldumount, 
   &jp_sys_oldumount, &krp_sys_oldumount, TRACE_OLDUMOUNT},  
  
  {"sys_readlink", trace_sys_readlink, ret_handler_readlink, 
   &jp_sys_readlink, &krp_sys_readlink, TRACE_READLINK},
  {"sys_read", trace_sys_read, ret_handler_read, 
   &jp_sys_read, &krp_sys_read, TRACE_READ},
  {"sys_readahead", trace_sys_readahead, ret_handler_readahead, 
   &jp_sys_readahead, &krp_sys_readahead, TRACE_READAHEAD},  
  {"sys_readv", trace_sys_readv, ret_handler_readv, 
   &jp_sys_readv, &krp_sys_readv, TRACE_READV},
  {"sys_pread64", trace_sys_pread64, ret_handler_pread64, 
   &jp_sys_pread64, &krp_sys_pread64, TRACE_PREAD64},
  
  {"sys_sync", trace_sys_sync, ret_handler_sync, 
   &jp_sys_sync, &krp_sys_sync, TRACE_SYNC},
  {"sys_fsync", trace_sys_fsync, ret_handler_fsync, 
   &jp_sys_fsync, &krp_sys_fsync, TRACE_FSYNC},
  {"sys_fdatasync", trace_sys_fdatasync, ret_handler_fdatasync, 
   &jp_sys_fdatasync, &krp_sys_fdatasync, TRACE_FDATASYNC},

  {"sys_stat", trace_sys_stat, ret_handler_stat, 
   &jp_sys_stat, &krp_sys_stat, TRACE_STAT},
  {"sys_statfs", trace_sys_statfs, ret_handler_statfs, 
   &jp_sys_statfs, &krp_sys_statfs, TRACE_STATFS},
  {"sys_statfs64", trace_sys_statfs64, ret_handler_statfs64, 
   &jp_sys_statfs64, &krp_sys_statfs64, TRACE_STATFS64},
  {"sys_fstatfs", trace_sys_fstatfs, ret_handler_fstatfs, 
   &jp_sys_fstatfs, &krp_sys_fstatfs, TRACE_FSTATFS},
  {"sys_fstatfs64", trace_sys_fstatfs64, ret_handler_fstatfs64, 
   &jp_sys_fstatfs64, &krp_sys_fstatfs64, TRACE_FSTATFS64},
  {"sys_lstat", trace_sys_lstat, ret_handler_lstat, 
   &jp_sys_lstat, &krp_sys_lstat, TRACE_LSTAT},
  {"sys_fstat", trace_sys_fstat, ret_handler_fstat, 
   &jp_sys_fstat, &krp_sys_fstat, TRACE_FSTAT},
  {"sys_newstat", trace_sys_newstat, ret_handler_newstat, 
   &jp_sys_newstat, &krp_sys_newstat, TRACE_NEWSTAT},
  {"sys_newlstat", trace_sys_newlstat, ret_handler_newlstat, 
   &jp_sys_newlstat, &krp_sys_newlstat, TRACE_NEWLSTAT},
  {"sys_newfstat", trace_sys_newfstat, ret_handler_newfstat, 
   &jp_sys_newfstat, &krp_sys_newfstat, TRACE_NEWFSTAT},
  {"sys_ustat", trace_sys_ustat, ret_handler_ustat, 
   &jp_sys_ustat, &krp_sys_ustat, TRACE_USTAT},

  {"sys_brk", trace_sys_brk, ret_handler_brk, 
   &jp_sys_brk, &krp_sys_brk, TRACE_BRK},

  {"sys_truncate", trace_sys_truncate, ret_handler_truncate, 
   &jp_sys_truncate, &krp_sys_truncate, TRACE_TRUNCATE},
  {"sys_ftruncate", trace_sys_ftruncate, ret_handler_ftruncate, 
   &jp_sys_ftruncate, &krp_sys_ftruncate, TRACE_FTRUNCATE},

  {"sys_bdflush", trace_sys_bdflush, ret_handler_bdflush, 
   &jp_sys_bdflush, &krp_sys_bdflush, TRACE_BDFLUSH},

  {"sys_setxattr", trace_sys_setxattr, ret_handler_setxattr, 
   &jp_sys_setxattr, &krp_sys_setxattr, TRACE_SETXATTR},
  {"sys_lsetxattr", trace_sys_lsetxattr, ret_handler_lsetxattr, 
   &jp_sys_lsetxattr, &krp_sys_lsetxattr, TRACE_LSETXATTR},
  {"sys_fsetxattr", trace_sys_fsetxattr, ret_handler_fsetxattr, 
   &jp_sys_fsetxattr, &krp_sys_fsetxattr, TRACE_FSETXATTR},

  {"sys_getxattr", trace_sys_getxattr, ret_handler_getxattr, 
   &jp_sys_getxattr, &krp_sys_getxattr, TRACE_GETXATTR},
  {"sys_lgetxattr", trace_sys_lgetxattr, ret_handler_lgetxattr, 
   &jp_sys_lgetxattr, &krp_sys_lgetxattr, TRACE_LGETXATTR},
  {"sys_fgetxattr", trace_sys_fgetxattr, ret_handler_fgetxattr, 
   &jp_sys_fgetxattr, &krp_sys_fgetxattr, TRACE_FGETXATTR},

  {"sys_listxattr", trace_sys_listxattr, ret_handler_listxattr, 
   &jp_sys_listxattr, &krp_sys_listxattr, TRACE_LISTXATTR},
  {"sys_llistxattr", trace_sys_llistxattr, ret_handler_llistxattr, 
   &jp_sys_llistxattr, &krp_sys_llistxattr, TRACE_LLISTXATTR},
  {"sys_flistxattr", trace_sys_flistxattr, ret_handler_flistxattr, 
   &jp_sys_flistxattr, &krp_sys_flistxattr, TRACE_FLISTXATTR},

  {"sys_removexattr", trace_sys_removexattr, ret_handler_removexattr, 
   &jp_sys_removexattr, &krp_sys_removexattr, TRACE_REMOVEXATTR},
  {"sys_lremovexattr", trace_sys_lremovexattr, ret_handler_lremovexattr, 
   &jp_sys_lremovexattr, &krp_sys_lremovexattr, TRACE_LREMOVEXATTR},
  {"sys_fremovexattr", trace_sys_fremovexattr, ret_handler_fremovexattr, 
   &jp_sys_fremovexattr, &krp_sys_fremovexattr, TRACE_FREMOVEXATTR},

  {"sys_rename", trace_sys_rename, ret_handler_rename, 
   &jp_sys_rename, &krp_sys_rename, TRACE_RENAME},

  {"sys_mknod", trace_sys_mknod, ret_handler_mknod, 
   &jp_sys_mknod, &krp_sys_mknod, TRACE_MKNOD},

  {"sys_fcntl", trace_sys_fcntl, ret_handler_fcntl, 
   &jp_sys_fcntl, &krp_sys_fcntl, TRACE_FCNTL},
  {"sys_dup", trace_sys_dup, ret_handler_dup, 
   &jp_sys_dup, &krp_sys_dup, TRACE_DUP},
  {"sys_dup2", trace_sys_dup2, ret_handler_dup2, 
   &jp_sys_dup2, &krp_sys_dup2, TRACE_DUP2},

  {"sys_ioctl", trace_sys_ioctl, ret_handler_ioctl, 
   &jp_sys_ioctl, &krp_sys_ioctl, TRACE_IOCTL},

  {"sys_flock", trace_sys_flock, ret_handler_flock, 
   &jp_sys_flock, &krp_sys_flock, TRACE_FLOCK},

  {"sys_getcwd", trace_sys_getcwd, ret_handler_getcwd, 
   &jp_sys_getcwd, &krp_sys_getcwd, TRACE_GETCWD},

  {"sys_mkdir", trace_sys_mkdir, ret_handler_mkdir, 
   &jp_sys_mkdir, &krp_sys_mkdir, TRACE_MKDIR},
  {"sys_chdir", trace_sys_chdir, ret_handler_chdir, 
   &jp_sys_chdir, &krp_sys_chdir, TRACE_CHDIR},
  {"sys_fchdir", trace_sys_fchdir, ret_handler_fchdir, 
   &jp_sys_fchdir, &krp_sys_fchdir, TRACE_FCHDIR},
  {"sys_rmdir", trace_sys_rmdir, ret_handler_rmdir, 
   &jp_sys_rmdir, &krp_sys_rmdir, TRACE_RMDIR},

  {"sys_getdents", trace_sys_getdents, ret_handler_getdents, 
   &jp_sys_getdents, &krp_sys_getdents, TRACE_GETDENTS},
  {"sys_getdents64", trace_sys_getdents64, ret_handler_getdents64, 
   &jp_sys_getdents64, &krp_sys_getdents64, TRACE_GETDENTS64},

  {"sys_quotactl", trace_sys_quotactl, ret_handler_quotactl, 
   &jp_sys_quotactl, &krp_sys_quotactl, TRACE_QUOTACTL},

  {"sys_write", trace_sys_write, ret_handler_write, 
   &jp_sys_write, &krp_sys_write, TRACE_WRITE},
  {"sys_writev", trace_sys_writev, ret_handler_writev, 
   &jp_sys_writev, &krp_sys_writev, TRACE_WRITEV},
  {"sys_pwrite64", trace_sys_pwrite64, ret_handler_pwrite64, 
   &jp_sys_pwrite64, &krp_sys_pwrite64, TRACE_PWRITE64},

  {"sys_lseek", trace_sys_lseek, ret_handler_lseek, 
   &jp_sys_lseek, &krp_sys_lseek, TRACE_LSEEK},
  {"sys_llseek", trace_sys_llseek, ret_handler_llseek, 
   &jp_sys_llseek, &krp_sys_llseek, TRACE_LLSEEK},

  {"sys_utime", trace_sys_utime, ret_handler_utime, 
   &jp_sys_utime, &krp_sys_utime, TRACE_UTIME},
  {"sys_utimes", trace_sys_utimes, ret_handler_utimes, 
   &jp_sys_utimes, &krp_sys_utimes, TRACE_UTIMES},

  {"sys_uselib", trace_sys_uselib, ret_handler_uselib, 
   &jp_sys_uselib, &krp_sys_uselib, TRACE_USELIB},

#ifdef CONFIG_UID16
  {"sys_chown16", trace_sys_chown16, ret_handler_chown16, 
   &jp_sys_chown16, &krp_sys_chown16, TRACE_CHOWN16},
  {"sys_lchown16", trace_sys_lchown16, ret_handler_lchown16, 
   &jp_sys_lchown16, &krp_sys_lchown16, TRACE_LCHOWN16},
  {"sys_fchown16", trace_sys_fchown16, ret_handler_fchown16, 
   &jp_sys_fchown16, &krp_sys_fchown16, TRACE_FCHOWN16},
#endif

#if BITS_PER_LONG == 32
  {"sys_stat64", trace_sys_stat64, ret_handler_stat64, 
   &jp_sys_stat64, &krp_sys_stat64, TRACE_STAT64},
  {"sys_fstat64", trace_sys_fstat64, ret_handler_fstat64, 
   &jp_sys_fstat64, &krp_sys_fstat64, TRACE_FSTAT64},
  {"sys_lstat64", trace_sys_lstat64, ret_handler_lstat64, 
   &jp_sys_lstat64, &krp_sys_lstat64, TRACE_LSTAT64},

  {"sys_truncate64", trace_sys_truncate64, ret_handler_truncate64, 
   &jp_sys_truncate64, &krp_sys_truncate64, TRACE_TRUNCATE64},
  {"sys_ftruncate64", trace_sys_ftruncate64, ret_handler_ftruncate64, 
   &jp_sys_ftruncate64, &krp_sys_ftruncate64, TRACE_FTRUNCATE64},

  {"sys_fcntl64", trace_sys_fcntl64, ret_handler_fcntl64, 
   &jp_sys_fcntl64, &krp_sys_fcntl64, TRACE_FCNTL64},

#endif
				 
   {"", 0, 0, 0, 0, -1}
};

/*******************Procfile section************************************/
struct proc_dir_entry* proc_directory;
static struct proc_dir_entry* proc_input_file;

/***********************************************************************/
/****************Tracing Data Procfiles*********************************/
/***********************************************************************/
int
procfile_trace_read(char* buffer,
		    char** buffer_location,
		    off_t offset, 
		    int buffer_length, 
		    int* eof, 
		    void* data)
{
  int i_ret;
  if (offset > 0)
  {
    i_ret = 0;
  }
  else
  {
    i_ret = read_buffer_chain_data(buffer, 
				   buffer_length, 
				   head, 
				   *((long*)(data)));
  }
  return i_ret;
}

/***********************************************************************/
int
procfile_trace_write(struct file *file, 
		     const char *buffer, 
		     unsigned long count, 
		     void *data)
{
  return -EFAULT;
}

/***********************************************************************/
/****************Trace-control Procfile*********************************/
/***********************************************************************/
int
procfile_input_read(char* buffer,
		    char** buffer_location,
		    off_t offset, 
		    int buffer_length, 
		    int* eof, 
		    void* data)
{
  return -EFAULT;
}

/***********************************************************************/
int
procfile_input_write(struct file *file, 
		     const char *buffer, 
		     unsigned long count, 
		     void *data)
{
  char pc_pid[16];
  int pid;
  char pid_switch;
  int i, j;  /* used as small local vars */
  char* k;
  char pc_list[2048];

  struct buffer_chain_node* proc_buf;


  if ( copy_from_user(&pid_switch, buffer, 1) )
    return -EFAULT;

  i = 0;
  if ( copy_from_user(pc_pid, buffer+1, 1) )
    return -EFAULT;
  while ( pc_pid[i] != ' ' && pc_pid[i] != '\0' && i+1 != count )
  {
    i++;
    if ( copy_from_user(pc_pid+i, buffer+i+1, 1) )
      return -EFAULT;
  }
  i++;
  pc_pid[i] = '\0';

  pid = simple_strtol(pc_pid, 0, 10);
  /*
  if ( count-i > 2048 || count - i <= 0)
    return -EFAULT;

  if ( copy_from_user(pc_list, buffer+i, count-i) )
    return -EFAULT;
  */
  switch(pid_switch)
  {
  case '+':
    proc_buf = procfile_init(pid);
    if ( proc_buf == 0 )
    {
      printk(KERN_INFO "I/O tracer: failed turning on for pid:%d.\n", pid);
    }
    else
    {
      printk(KERN_INFO "I/O tracer: on for pid:%d.\n", pid);
    
      /* first turn "on" or "off" all, with the idea that all is a general 
	 mapping and then users can choose to trace/not trace only 
	 specific processes*//*
      k = strstr(pc_list, "all");
      if ( k != NULL )
      {
	if ( k <= pc_list )
	{
	  printk(KERN_INFO "I/O tracer: parsing error in turning on:%d.\n", pid);
	  unlock_all(head);
	  procfile_remove(pid);
	  return -1;
	}

	switch ((k-1)[0])
	{
	case '+':
	  for ( j = 0; j < 128; j++ )
	    proc_buf->a_kprobes[j] = 1;
	  break;
	case '-':
	  for ( j = 0; j < 128; j++ )
	    proc_buf->a_kprobes[j] = 0;
	  break;
	default:
	  printk(KERN_INFO "I/O tracer: parsing error in turning on:%d.\n", pid);
	  unlock_all(head);
	  procfile_remove(pid);
	  return -1;
	}
      }
      
      i = 0;
      while ( (kprobe_table[i]).name[0] != '\0' )
      {
	k = strstr(pc_list, kprobe_table[i].name);
	if ( k != NULL )
	{
	  if ( k <= pc_list )
	  {
	    printk(KERN_INFO "I/O tracer: parsing error in turning on:%d.\n", pid);
	    unlock_all(head);
	    procfile_remove(pid);
	    return -1;
	  }

	  switch ((k-1)[0])
	  {
	  case '+':
	    proc_buf->a_kprobes[kprobe_table[i].index] = 1;
	    break;
	  case '-':
	    proc_buf->a_kprobes[kprobe_table[i].index] = 1;
	    break;
	  default:
	    printk(KERN_INFO "I/O tracer: parsing error in turning on:%d.\n", pid);
	    unlock_all(head);
	    procfile_remove(pid);
	    return -1;
	  }
	}
      }			     */
    }
    //unlock_all(head);
    break;
  case '-':
    procfile_remove(pid);
    printk(KERN_INFO "I/O tracer: off for pid:%d.\n", pid);
    break;
  default:
    printk(KERN_INFO "I/O tracer: Invalid on/off switch. Use + or -");
    return -1;
  }
  
  return count;

}

/***********************************************************************/
/******************Kernel startup and cleanup***************************/
static int
register_io_probe(struct kprobe_table_entry* kte)
{
  int i_ret = 0;

  kte->pjp->kp.symbol_name = kte->name;
  kte->pjp->entry = kte->entry;

  i_ret = register_jprobe(kte->pjp);
  if ( i_ret < 0 )
  {
    printk(KERN_INFO "register kprobe %s failed, returned %d\n", 
	   kte->name,
	   i_ret);
    return -1;
  }

  kte->pkrp->handler = kte->ret;
  kte->pkrp->kp.symbol_name = kte->name;
  kte->pkrp->maxactive=200;
  
  i_ret = register_kretprobe(kte->pkrp);
  if ( i_ret < 0 )
  {
    printk(KERN_INFO "register kret_prob %s failed, returned %d\n", 
	   kte->name, 
	   i_ret);
    return -1;
  }
  
  return 0;
}

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
static int 
__init testmodule_init(void)
{
  int i = 0;
  printk(KERN_INFO "I/O tracing turning on.\n");
  printk(KERN_INFO "Page sizes are %lu\n", PAGE_SIZE);

  head = kmalloc(sizeof(struct buffer_chain_node), GFP_KERNEL);
  if (head == 0 )
    return -ENOMEM;
  head->bc = 0;
  head->next = NULL;
  head->pid = 0;
  head->lock = kmalloc(sizeof(spinlock_t), GFP_KERNEL);
  if (head->lock == 0)
    return -ENOMEM;
  *(head->lock) = SPIN_LOCK_UNLOCKED;

  proc_directory = proc_mkdir(PROCFS_DIRECTORY_NAME, NULL);

  proc_input_file =
    create_proc_entry("trace_control", 0644, proc_directory);
  if ( proc_input_file == NULL )
  {
    return -ENOMEM;
  }

  proc_input_file->read_proc = procfile_input_read;
  proc_input_file->write_proc = procfile_input_write;
  proc_input_file->owner = THIS_MODULE;
  proc_input_file->mode = S_IFREG | S_IRUGO;
  proc_input_file->uid = 0;
  proc_input_file->gid = 0;

  i = 0;
  while ( kprobe_table[i].entry != 0 )
  {
    if ( register_io_probe(&(kprobe_table[i])) < 0 )
      return -1;

    i++;
  }  

  return 0;
}

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
static void __exit testmodule_exit(void)
{
  int i;

  printk(KERN_INFO "I/O tracing turning off.\n");

  i = 0;
  while ( kprobe_table[i].entry != 0 )
  {
    unregister_kretprobe(kprobe_table[i].pkrp);
    unregister_jprobe(kprobe_table[i].pjp);
    i++;
  }

  destroy_buffers(head);

  remove_proc_entry(PROCFS_DIRECTORY_NAME, &proc_root);
}

/***********************************************************************/

module_init(testmodule_init);
module_exit(testmodule_exit);
